@@ -41,6 +41,9 @@
 #include "empathy-tube-handler.h"
 #include "empathy-contact-factory.h"
 #include "empathy-tp-group.h"
+#include "empathy-tp-file.h"
+#include "empathy-chatroom-manager.h"
+#include "empathy-utils.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_DISPATCHER
 #include <libempathy/empathy-debug.h>
@@ -51,6 +54,7 @@ typedef struct {
 	gpointer        token;
 	MissionControl *mc;
 	GSList         *tubes;
+  EmpathyChatroomManager *chatroom_mgr;
 } EmpathyDispatcherPriv;
 
 G_DEFINE_TYPE (EmpathyDispatcher, empathy_dispatcher, G_TYPE_OBJECT);
@@ -389,6 +393,59 @@ dispatcher_connection_invalidated_cb (TpConnection  *connection,
 	}
 }
 
+typedef struct
+{
+  EmpathyDispatcher *self;
+  EmpathyChatroom *chatroom;
+} dispatcher_connection_invalidated_cb_ctx;
+
+static dispatcher_connection_invalidated_cb_ctx *
+dispatcher_connection_invalidated_cb_ctx_new (EmpathyDispatcher *dispatcher,
+                                              EmpathyChatroom *chatroom)
+{
+  dispatcher_connection_invalidated_cb_ctx *ctx;
+
+  ctx = g_slice_new (dispatcher_connection_invalidated_cb_ctx);
+
+  ctx->self = g_object_ref (dispatcher);
+  ctx->chatroom = g_object_ref (chatroom);
+
+  return ctx;
+}
+
+static void
+dispatcher_connection_invalidated_cb_ctx_free (
+    dispatcher_connection_invalidated_cb_ctx *ctx)
+{
+  g_object_unref (ctx->self);
+  g_object_unref (ctx->chatroom);
+
+  g_slice_free (dispatcher_connection_invalidated_cb_ctx, ctx);
+}
+
+static void dispatcher_chatroom_invalidated_cb (
+    TpProxy *channel,
+    guint domain,
+    gint code,
+    gchar *message,
+    dispatcher_connection_invalidated_cb_ctx *ctx)
+{
+  EmpathyDispatcherPriv *priv = GET_PRIV (ctx->self);
+  gboolean favorite;
+
+  g_object_get (ctx->chatroom, "favorite", &favorite, NULL);
+
+  if (favorite)
+    {
+      /* Chatroom is in favorites so don't remove it from the manager */
+      g_object_set (ctx->chatroom, "tp-channel", NULL, NULL);
+    }
+  else
+    {
+      empathy_chatroom_manager_remove (priv->chatroom_mgr, ctx->chatroom);
+    }
+}
+
 static void
 dispatcher_connection_new_channel_cb (TpConnection *connection,
 				      const gchar  *object_path,
@@ -400,6 +457,7 @@ dispatcher_connection_new_channel_cb (TpConnection *connection,
 				      GObject      *object)
 {
 	EmpathyDispatcher *dispatcher = EMPATHY_DISPATCHER (object);
+  EmpathyDispatcherPriv *priv = GET_PRIV (dispatcher);
 	TpChannel         *channel;
 	gpointer           had_channels;
 
@@ -418,6 +476,57 @@ dispatcher_connection_new_channel_cb (TpConnection *connection,
 		dispatcher_tubes_handle_channel (dispatcher, channel);
 	}
 
+  if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TEXT) &&
+      handle_type == TP_HANDLE_TYPE_ROOM)
+    {
+      /* Add the chatroom to the chatroom manager */
+      EmpathyChatroom *chatroom;
+      GArray *handles;
+      gchar **room_ids;
+      MissionControl *mc;
+      McAccount *account;
+      dispatcher_connection_invalidated_cb_ctx *ctx;
+
+      handles = g_array_sized_new (FALSE, FALSE, sizeof (TpHandle), 1);
+      g_array_append_val (handles, handle);
+
+      tp_cli_connection_run_inspect_handles (connection, -1,
+          TP_HANDLE_TYPE_ROOM, handles, &room_ids, NULL, NULL);
+
+      mc = empathy_mission_control_new ();
+      account = mission_control_get_account_for_tpconnection (mc, connection,
+          NULL);
+
+      chatroom = empathy_chatroom_manager_find (priv->chatroom_mgr, account,
+          room_ids[0]);
+      if (chatroom == NULL)
+        {
+          chatroom = empathy_chatroom_new (account);
+          empathy_chatroom_set_name (chatroom, room_ids[0]);
+          empathy_chatroom_set_room (chatroom, room_ids[0]);
+          empathy_chatroom_manager_add (priv->chatroom_mgr, chatroom);
+        }
+      else
+        {
+          g_object_ref (chatroom);
+        }
+
+      g_object_set (chatroom, "tp-channel", channel, NULL);
+
+      ctx = dispatcher_connection_invalidated_cb_ctx_new (dispatcher, chatroom);
+
+      g_signal_connect_data (channel, "invalidated",
+          G_CALLBACK (dispatcher_chatroom_invalidated_cb), ctx,
+          (GClosureNotify) dispatcher_connection_invalidated_cb_ctx_free, 0);
+
+      g_free (room_ids[0]);
+      g_free (room_ids);
+      g_array_free (handles, TRUE);
+      g_object_unref (mc);
+      g_object_unref (account);
+      g_object_unref (chatroom);
+    }
+
 	if (suppress_handler) {
 		g_signal_emit (dispatcher, signals[DISPATCH_CHANNEL], 0, channel);
 	} else {
@@ -579,6 +688,8 @@ dispatcher_finalize (GObject *object)
 	g_slist_free (priv->tubes);
 
 	g_hash_table_destroy (priv->connections);
+
+  g_object_unref (priv->chatroom_mgr);
 }
 
 static void
@@ -642,6 +753,8 @@ empathy_dispatcher_init (EmpathyDispatcher *dispatcher)
 		g_object_unref (l->data);
 	}
 	g_list_free (accounts);
+
+  priv->chatroom_mgr = empathy_chatroom_manager_new (NULL);
 }
 
 EmpathyDispatcher *
@@ -816,3 +929,161 @@ empathy_dispatcher_chat_with_contact_id (McAccount   *account,
 	g_object_unref (factory);
 }
 
+typedef struct {
+	GFile *gfile;
+	TpHandle handle;
+} FileChannelRequest;
+
+static void
+tp_file_state_notify_cb (EmpathyTpFile *tp_file)
+{
+	EmpFileTransferState state;
+
+	state = empathy_tp_file_get_state (tp_file, NULL);
+	if (state == EMP_FILE_TRANSFER_STATE_COMPLETED ||
+	    state == EMP_FILE_TRANSFER_STATE_CANCELLED) {
+		DEBUG ("Transfer is done, unref the object");
+		g_object_unref (tp_file);
+	}
+}
+
+static void
+file_channel_create_cb (TpConnection *connection,
+			const gchar  *object_path,
+			GHashTable   *properties,
+			const GError *error,
+			gpointer      user_data,
+			GObject      *weak_object)
+{
+	TpChannel *channel;
+	EmpathyTpFile *tp_file;
+	FileChannelRequest *request = (FileChannelRequest *) user_data;
+
+	if (error) {
+		DEBUG ("Couldn't request channel: %s", error->message);
+		return;
+	}
+
+	channel = tp_channel_new (connection,
+				 object_path,
+				 EMP_IFACE_CHANNEL_TYPE_FILE_TRANSFER,
+				 TP_HANDLE_TYPE_CONTACT,
+				 request->handle,
+				 NULL);
+
+	/* We give the ref to the callback, it is responsible to unref the
+	 * object once the transfer is done. */
+	tp_file = empathy_tp_file_new (channel);
+	empathy_tp_file_offer (tp_file, request->gfile, NULL);
+	g_signal_connect (tp_file, "notify::state",
+			  G_CALLBACK (tp_file_state_notify_cb),
+			  NULL);
+
+	g_object_unref (request->gfile);
+	g_slice_free (FileChannelRequest, request);
+	g_object_unref (channel);
+}
+
+void
+empathy_dispatcher_send_file (EmpathyContact *contact,
+			      GFile          *gfile)
+{
+	MissionControl     *mc;
+	McAccount          *account;
+	TpConnection       *connection;
+	guint               handle;
+	FileChannelRequest *request;
+	GHashTable         *args;
+	GValue             *value;
+	GFileInfo          *info;
+	gchar              *filename;
+	GTimeVal            last_modif;
+	GError             *error = NULL;
+
+	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+	g_return_if_fail (G_IS_FILE (gfile));
+
+	info = g_file_query_info (gfile,
+				  G_FILE_ATTRIBUTE_STANDARD_SIZE ","
+				  G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE ","
+				  G_FILE_ATTRIBUTE_TIME_MODIFIED,
+				  0, NULL, &error);
+
+	if (error) {
+		DEBUG ("Can't get info about the file: %s", error->message);
+		g_clear_error (&error);
+		return;
+	}
+
+	mc = empathy_mission_control_new ();
+	account = empathy_contact_get_account (contact);
+	connection = mission_control_get_tpconnection (mc, account, NULL);
+	handle = empathy_contact_get_handle (contact);
+
+	request = g_slice_new0 (FileChannelRequest);
+	request->gfile = g_object_ref (gfile);
+	request->handle = handle;
+
+	filename = g_file_get_basename (request->gfile);
+	tp_connection_run_until_ready (connection, FALSE, NULL, NULL);
+
+	DEBUG ("Sending %s from a stream to %s (size %lli, content-type %s)",
+	       filename, empathy_contact_get_name (contact),
+	       g_file_info_get_size (info),
+	       g_file_info_get_content_type (info));
+
+	args = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
+				      (GDestroyNotify) tp_g_value_slice_free);
+
+	/* org.freedesktop.Telepathy.Channel.ChannelType */
+	value = tp_g_value_slice_new (G_TYPE_STRING);
+	g_value_set_string (value, EMP_IFACE_CHANNEL_TYPE_FILE_TRANSFER);
+	g_hash_table_insert (args, TP_IFACE_CHANNEL ".ChannelType", value);
+
+	/* org.freedesktop.Telepathy.Channel.TargetHandleType */
+	value = tp_g_value_slice_new (G_TYPE_UINT);
+	g_value_set_uint (value, TP_HANDLE_TYPE_CONTACT);
+	g_hash_table_insert (args, TP_IFACE_CHANNEL ".TargetHandleType", value);
+
+	/* org.freedesktop.Telepathy.Channel.TargetHandle */
+	value = tp_g_value_slice_new (G_TYPE_UINT);
+	g_value_set_uint (value, handle);
+	g_hash_table_insert (args, TP_IFACE_CHANNEL ".TargetHandle", value);
+
+	/* org.freedesktop.Telepathy.Channel.Type.FileTransfer.ContentType */
+	value = tp_g_value_slice_new (G_TYPE_STRING);
+	g_value_set_string (value, g_file_info_get_content_type (info));
+	g_hash_table_insert (args,
+		EMP_IFACE_CHANNEL_TYPE_FILE_TRANSFER ".ContentType", value);
+
+	/* org.freedesktop.Telepathy.Channel.Type.FileTransfer.Filename */
+	value = tp_g_value_slice_new (G_TYPE_STRING);
+	g_value_set_string (value, g_filename_display_basename (filename));
+	g_hash_table_insert (args,
+		EMP_IFACE_CHANNEL_TYPE_FILE_TRANSFER ".Filename", value);
+
+	/* org.freedesktop.Telepathy.Channel.Type.FileTransfer.Size */
+	value = tp_g_value_slice_new (G_TYPE_UINT64);
+	g_value_set_uint64 (value, g_file_info_get_size (info));
+	g_hash_table_insert (args,
+		EMP_IFACE_CHANNEL_TYPE_FILE_TRANSFER ".Size", value);
+
+	/* org.freedesktop.Telepathy.Channel.Type.FileTransfer.Date */
+	g_file_info_get_modification_time (info, &last_modif);
+	value = tp_g_value_slice_new (G_TYPE_UINT64);
+	g_value_set_uint64 (value, last_modif.tv_sec);
+	g_hash_table_insert (args,
+		EMP_IFACE_CHANNEL_TYPE_FILE_TRANSFER ".Date", value);
+
+	/* FIXME: Description ? */
+	/* FIXME: ContentHashType and ContentHash ? */
+
+	tp_cli_connection_interface_requests_call_create_channel (connection, -1,
+		args, file_channel_create_cb, request, NULL, NULL);
+
+	g_hash_table_destroy (args);
+	g_free (filename);
+	g_object_unref (mc);
+	g_object_unref (connection);
+}
+
